//
//  audioAL.m
//  breakout
//
//  Created by HANSON, PAUL  on 21/04/2010.
//  Copyright 2010 __MyCompanyName__. All rights reserved.
//

#import "audioAL.h"

@implementation audioAL

/* 
Grabs the device and context
Returns - bool
*/
-(bool)initAudioDevice
{
	//init the dictionary
	soundDictionary = [[NSMutableDictionary alloc] init];
	
	//open device
	oalDevice = alcOpenDevice(NULL);
	
	//check to see if we have grabbed it
	if (oalDevice)
	{
		//create context
		oalContext = alcCreateContext(oalDevice, NULL);
		//set the current context
		alcMakeContextCurrent(oalContext);
	}
	
	//check to see things have worked
	if ((!oalDevice) && (!oalContext))
	{
		printf("FAILED open creation");
		return NO;	
	}
	
	else
	{
		printf("Device Loaded\n");
		return YES;
	}

}

/* 
Loads the an audio clip
Returns - bool
*/
-(bool)loadAudio
{
	//set the clip path,name and type
	NSString* filename = [[NSBundle  mainBundle] pathForResource:@"chomp" ofType:@"caf"];
	//open the file
	AudioFileID fileID = [self openAudioFile:filename];
	//get the size of the file
	UInt32 fileSize = [self audioFileSize:fileID];
	
	//error check
	if ((!fileID) && (!fileSize))
	{
		printf("Failed to load sounds\n");
		return NO;
	}
	
	//assign the some data
	unsigned char * outData = malloc(fileSize);
	
	//get the bytes
	OSStatus result = noErr; //init
	result = AudioFileReadBytes(fileID, false, 0, &fileSize, outData);
	
	//finished with the file, close it
	AudioFileClose(fileID);
	
	if (result != 0)
	{
		NSLog(@"Cannot load %@\n", filename);
		return NO;
	}
	
	
	//create buffer
	alGenBuffers(1, &bufferID);
	//fill it with audio
	alBufferData(bufferID, AL_FORMAT_STEREO16, outData, fileSize, 44100);
	
	//save buffer, release later
	//[bufferStorageArray addObject:[NSNumber numberWithUnsignedInteger:bufferID]];
	
	//create source
	alGenSources(1, &sourceID);
	
	//attach buffer to source
	alSourcei(sourceID, AL_BUFFER, bufferID);
	
	//set the source preferences
	alSourcef(sourceID, AL_PITCH, 1.0f);
	alSourcef(sourceID,AL_GAIN, 1.0f);
	
	//loop it
	//alSourcei(sourceID, AL_LOOPING, AL_TRUE);
	
	//store for later use
	[soundDictionary setObject:[NSNumber numberWithUnsignedInt:sourceID] forKey:@"Chomp"];
	
	//clean up
	if(outData)
	{
		free(outData);
		outData = NULL;
	}

	
	return YES;
}

/* 
Opens an audio file
Parameter - NSString* 
Returns - AudioFileID
*/
-(AudioFileID) openAudioFile:(NSString*)filePath
{
	//return value
	AudioFileID outAFID;
	
	//create the path URL
	NSURL * afURL = [NSURL fileURLWithPath:filePath];
	
	//open the file and create a pointer to it
	OSStatus result = AudioFileOpenURL((CFURLRef)afURL, kAudioFileReadPermission, 0, &outAFID);
	
	//error check - result will be 0 is successful
	if (result != 0)
	{
		NSLog(@"Cannot load %@\n", filePath);
		result = 0; //keep it running
	}
	
	else
	{
		NSLog(@"Loaded %@\n", filePath);
	}
	
	//return pointer to the file
	return outAFID;
}

/* 
Gets the audio clip file size
Parameter - AudioFileID 
Returns - UInt32
*/
-(UInt32) audioFileSize:(AudioFileID) fileDescriptor;
{
	//init return value
	UInt64 outDataSize = 0;
	
	UInt32 thePropSize =sizeof(UInt64);
	
	//find out how much data is there
	OSStatus result = AudioFileGetProperty(fileDescriptor, kAudioFilePropertyAudioDataByteCount, &thePropSize, &outDataSize);
	
	//error check
	if (result !=0)
	{
		printf("Cannot get file size\n");
		result = 0; //keep it running
	}
	
	//return the file size
	return (UInt32) outDataSize;
}

/* 
Plays an audio clip stored in the dictionary
Parameter - NSString* 
*/
-(void) playSound:(NSString*) soundKey
{
	//find the value in the dictionary
	NSNumber *numVal = [soundDictionary objectForKey:soundKey];
	
	//make sure its passed a value
	if (numVal == nil)
	{
		return;
	}
	
	//Change the number to an int value
	NSUInteger playSourceID = [numVal unsignedIntValue];
	//play sound
	alSourcePlay(playSourceID);
}

/* 
Function not really needed. Only required for looping sounds 
Stops the audio clip from playing
Parameter - NSString* 
*/
-(void) stopSound:(NSString*) soundKey
{
	//find the value in the dictionary
	NSNumber *numVal = [soundDictionary objectForKey:soundKey];
	
	//make sure its passed a value
	if (numVal == nil)
	{
		return;
	}
	
	//Change the number to an int value
	NSUInteger stopSourceID = [numVal unsignedIntValue];
	//stop sound
	alSourcePlay(stopSourceID);
}

/* 
Cleans up the class
*/
-(void) cleanUpSound
{
	//clean the dictionary
	for (NSNumber * sourceNumber in [soundDictionary allValues])
	{
		NSUInteger cleanSourceID = [sourceNumber unsignedIntegerValue];
		alDeleteSources(1, &cleanSourceID);
	}
	
	//remove all object in dict
	[soundDictionary removeAllObjects];
	
	//delete buffers i dont have
	alDeleteBuffers(1, &bufferID);

	//destroy context
	alcDestroyContext(oalContext);
	//destroy device
	alcCloseDevice(oalDevice);
}


@end
