/*#include "gameLogic.h"

//init structs
snake player;
item fruit;
map world;

//pointers to structs
snake *p_player = &player;
item *p_fruit = &fruit;
map *p_world = &world;
CGRect tileRects[600];
CGRect *p_tileRect = &tileRects[600];

//inline functions
extern inline unsigned int getSpeed() {return player.speed;}
extern inline unsigned int getScore() {return playerScore;}
extern inline unsigned int getHighScore() {return highScore;}
extern void compareScores() {if (playerScore > highScore) highScore = playerScore;}
extern unsigned int getCurrentDirection() {return player.snakeDirection;}

//functions

/*
 Sets a random tile in the map to an item
 Parameter - item, map
 */
void createItem(item *i, map *m)
{
	//get a random x and y value	
	i->xPos = rand() / (RAND_MAX / TILE_WIDTH);
	i->yPos = rand() / (RAND_MAX / TILE_HEIGHT);
	
	//make sure its not on the snake
	while(m->tile[i->xPos][i->yPos] == SNAKE)
	{
		i->xPos = rand() / (RAND_MAX / TILE_WIDTH);
		i->yPos = rand() / (RAND_MAX / TILE_HEIGHT);
	}
	
	//choose the item type
	i->type = rand() / (RAND_MAX / ITEMTYPE_SIZE);
	
	//assign to map
	m->tile[i->xPos][i->yPos] = ITEM;
	
	//check
	printf("Fruit x is: %i \n", i->xPos);
	printf("Fruit y is: %i, \n", i->yPos);
};

/*
 Detects a collsions between the snake and an item or the snake and itself
 Parameter - snake, map
 */
int collisionDetection(snake *s, map *m)
{
	//check the snakes direction
	switch(s->snakeDirection)
	{
		//in each case check the corresponding tile
		/*
		 Return values
		 1 - item
		 2 - snake
		 0 - none
		 */
		case FACE_UP:
			if (m->tile[s->segment[0].xPos][s->segment[0].yPos - 1] == ITEM)
			{
				printf("HIT ITEM\n");
				return 1;
			}
			
			if (m->tile[s->segment[0].xPos][s->segment[0].yPos - 1] == SNAKE)
			{
				printf("HIT SNAKE\n");
				return 2;
			}
			
			else
			{
				return 0;
			}
		
			break;
				
		case FACE_DOWN:
			if (m->tile[s->segment[0].xPos][s->segment[0].yPos + 1] == ITEM)
			{
				printf("HIT ITEM\n");
				return 1;
			}
			
			if (m->tile[s->segment[0].xPos][s->segment[0].yPos + 1] == SNAKE)
			{
				printf("HIT SNAKE\n");
				return 2;
			}
			
			
			else
			{
				return 0;
			}
			
			break;
			
		case FACE_LEFT:
			if (m->tile[s->segment[0].xPos - 1][s->segment[0].yPos] == ITEM)
			{
				printf("HIT ITEM\n");
				return 1;
			}
			
			if (m->tile[s->segment[0].xPos - 1][s->segment[0].yPos] == SNAKE)
			{
				printf("HIT SNAKE\n");
				return 2;
			}
			
			else
			{
				return 0;
			}
			
			break;
			
		case FACE_RIGHT:
			if (m->tile[s->segment[0].xPos + 1][s->segment[0].yPos] == ITEM)
			{
				printf("HIT ITEM\n");
				return 1;
			}
			
			if (m->tile[s->segment[0].xPos + 1][s->segment[0].yPos] == SNAKE)
			{
				printf("HIT SNAKE\n");
				return 2;
			}
			
			else
			{
				return 0;
			}
			
			break;
			
		default:
			//no hit
			return 0;
			break;
	}
};
	
/*
 sets up the game variables
 */
void runLogic()
{
	//get random number
	srand ((unsigned short)time(NULL));

	//set up the map and screen rects
	int i = 0;
	for (int y = 0; y < TILE_HEIGHT; y++)
	{
		for (int x = 0; x < TILE_WIDTH; x++)
		{
			world.tile[x][y] = EMPTY;
			tileRects[i] = CGRectMake(x * TILEX, y * TILEY, TILEX, TILEY);
			i++;
		}
	}
	
	//load the images
	loadImage();
	
	//start position for snake
	player.xPos = rand() / (RAND_MAX / TILE_WIDTH);
	player.yPos = rand() / (RAND_MAX / TILE_HEIGHT);
	player.snakeDirection = FACE_RIGHT;
	
	//set the speed
	player.speed = 60 / playerStartSpeed; //fps over speed to give speed per frame
	
	//init body
	player.length = playerStartLength;
	int xOff = 0;
	int yOff = 0;
	for (int i = 0; i < player.length; i++)
	{
		//ensure its not off the screen
		if (player.xPos - xOff <= 1)
		{
			yOff++;
			xOff--;
		}
		
		//set the tile to snake		
		world.tile[player.xPos - xOff][player.yPos + yOff] = SNAKE;
		
		//body
		player.segment[i].xPos = player.xPos - xOff;
		player.segment[i].yPos = player.yPos + yOff;
		
		xOff++;
	}
	
	//i was sick of testing with 1 item... 
	for (int i = 0; i< numberOfItemsOnScreen; i++)
		createItem(p_fruit, p_world);
	
	//check map array
	for (int y = 0; y < TILE_HEIGHT; y++)
	{
		for (int x = 0; x < TILE_WIDTH; x++)
		{
			printf("%i ", world.tile[x][y]);
		}
		
		printf ("\n");
	}

	//check snake properties
	printf( "Snake head x is: %i \n", player.xPos);
	printf("Snake head y is: %i \n", player.yPos);
	printf("Snake direction: %i \n" ,player.snakeDirection);
	printf("Snake length: %i \n", player.length);
	printf("Snake speed is: %i \n", player.speed);
	
	//success
	printf("Set-Up Complete\n");
	
}

/*
 Updates the snakes position
 Parameter - int
 Return - hit
 */
int move(int value)
{
	//update snakes direction first
	player.snakeDirection = value;
	
	//check for collisions 
	int hit = collisionDetection(p_player, p_world);
		
	//collision with an item
	if (hit == 1)
	{
		//update score
		playerScore += SCORE_AWARD;
		//increase length
		player.length++;
		//create a new item
		createItem(p_fruit, p_world);
	}
	
	//update movement
	if (value == 0)
	{	
		//clear last segment
		world.tile[player.segment[player.length-1].xPos][player.segment[player.length-1].yPos] = EMPTY;
		
		//update body
		for (int x = player.length; x > 0 ;x--)
		{
			player.segment[x].xPos = player.segment[x-1].xPos;
			player.segment[x].yPos = player.segment[x-1].yPos;
		}
		
		//update head
		player.segment[0].yPos--;
		
		//wrap around map
		if (player.segment[0].yPos < 0)
		{
			player.segment[0].yPos = TILE_HEIGHT-1;
		}
		
		//make the tile a snake
		world.tile[player.segment[0].xPos][player.segment[0].yPos] = SNAKE;

	}
	
	
	if (value == 1)
	{
		//clear last segment
		world.tile[player.segment[player.length-1].xPos][player.segment[player.length-1].yPos] = EMPTY;
		
		//update body
		for (int x = player.length; x > 0 ;x--)
		{
			
			player.segment[x].xPos = player.segment[x-1].xPos;
			player.segment[x].yPos = player.segment[x-1].yPos;
			
		}
		
		//update head
		player.segment[0].yPos++;
		
		//wrap around
		if (player.segment[0].yPos > TILE_HEIGHT - 1)
		{
			player.segment[0].yPos = 0;
		}
		
		//make tile a snake
		world.tile[player.segment[0].xPos][player.segment[0].yPos] = SNAKE;

	}
	
	if (value == 2)
	{		
		//clear last segment
		world.tile[player.segment[player.length-1].xPos][player.segment[player.length-1].yPos] = EMPTY;
		
		//update body
		for (int x = player.length; x > 0 ;x--)
		{
			player.segment[x].xPos = player.segment[x-1].xPos;
			player.segment[x].yPos = player.segment[x-1].yPos;
			
		}
		
		//update head
		player.segment[0].xPos--;
		
		//wrap around
		if (player.segment[0].xPos < 0)
		{
			player.segment[0].xPos = TILE_WIDTH-1;
		}
		
		//make tile a snake
		world.tile[player.segment[0].xPos][player.segment[0].yPos] = SNAKE;

	}
	
	if (value == 3)
	{	
		//clear last segment
		world.tile[player.segment[player.length-1].xPos][player.segment[player.length-1].yPos] = EMPTY;
		
		//update body
		for (int x = player.length; x > 0 ;x--)
		{
			player.segment[x].xPos = player.segment[x-1].xPos;
			player.segment[x].yPos = player.segment[x-1].yPos;
		}
		
		//update head
		player.segment[0].xPos++;
		
		//wrap around
		if (player.segment[0].xPos > TILE_WIDTH - 1)
		{
			player.segment[0].xPos = 0;
		}
		
		//make the head a snake
		world.tile[player.segment[0].xPos][player.segment[0].yPos] = SNAKE;

	}

	return hit;
}

/*
 Renders the game map
 Parameters - Map, CGRect, CGContextRef
 */
void renderMap(map *m, CGRect *r, CGContextRef CG)
{
	//keeps track of the current tile
	int i = 0;
	
	//loop through the map
	for (int y = 0; y < TILE_HEIGHT; y++)
	{
		for (int x = 0; x < TILE_WIDTH; x++)
		{
			//check the tile and draw the corresponding image
			if (m->tile[x][y] == EMPTY)
			{
				CGContextDrawImage(CG, tileRects[i], emptyImage);
			}
						
			if (m->tile[x][y] == SNAKE)
			{
				CGContextDrawImage(CG, tileRects[i], snakeImage);
			}
						
			if (m->tile[x][y] == ITEM)
			{
				CGContextDrawImage(CG, tileRects[i], strawberryImage);
			}
						
			i++;
		}
	}		
}

/*
 Calls render map
 Parameters - CGContextRef
 */
void render(CGContextRef ref)
{
	renderMap(p_world, p_tileRect, ref);
}

/*
 Load a PNG sprite map from file and splits up the sprites
 */
void loadImage()
{
	//create the ref for the image
	CGImageRef spriteMap;

	//Bundle Reference
    CFBundleRef mainBundle = CFBundleGetMainBundle();
	
	//image file name
    CFStringRef imageFileName = CFStringCreateWithCString (NULL, "Sprites", kCFStringEncodingUTF8);
	
	//image URL
	CFURLRef imageURL = CFBundleCopyResourceURL(mainBundle, imageFileName, CFSTR("png"), NULL);
	
	//dont need the filename anymore
    CFRelease(imageFileName);
	
    //data from image
    CGDataProviderRef provider = CGDataProviderCreateWithURL(imageURL);
	
	//dont need the URL anymore
    CFRelease(imageURL);
	
	//Create image
	//data, colour map, interpolation, default gfx context
    spriteMap = CGImageCreateWithPNGDataProvider (provider, NULL, true, kCGRenderingIntentDefault);
	
	//bye bye provider data
    CGDataProviderRelease (provider);
		
	//split it up
	CGRect strawberryArea = CGRectMake(0, 0, 16, 16);
	CGRect appleArea = CGRectMake (0, 16, 16, 16);
	CGRect snakeArea = CGRectMake (0, 32, 16, 16);
	CGRect starArea = CGRectMake(0, 48, 16, 16);
	CGRect emptyArea = CGRectMake (0, 64, 16, 16);
	
	//assign
	strawberryImage = CGImageCreateWithImageInRect(spriteMap,strawberryArea);
	appleImage = CGImageCreateWithImageInRect (spriteMap, appleArea);
	snakeImage = CGImageCreateWithImageInRect (spriteMap, snakeArea);
	starImage = CGImageCreateWithImageInRect(spriteMap, starArea);
	emptyImage = CGImageCreateWithImageInRect (spriteMap, emptyArea);
	
	//bye bye sprite map
	CGImageRelease(spriteMap);
	
}

/*
Resets the Game
*/
void reset()
{	
	//reset map array back to empty
	for (char y = 0; y < TILE_HEIGHT; y++)
	{
		for (char x = 0; x < TILE_WIDTH; x++)
		{
			world.tile[x][y] = EMPTY;
		}
	}
	
	//start position for snake
	player.xPos = rand() / (RAND_MAX / TILE_WIDTH);
	player.yPos = rand() / (RAND_MAX / TILE_HEIGHT);
	player.snakeDirection = FACE_RIGHT;
		
	//reset the length
	player.length = playerStartLength;
	
	//init the snake
	int xOff = 0;
	int yOff = 0;
	for (int i = 0; i < player.length; i++)
	{
		//ensure its not off the screen
		if (player.xPos - xOff <= 1)
		{
			yOff++;
			xOff--;
		}
		
		world.tile[player.xPos - xOff][player.yPos + yOff] = SNAKE;
		
		//body
		player.segment[i].xPos = player.xPos - xOff;
		player.segment[i].yPos = player.yPos + yOff;
		
		xOff++;
	}
	
	//init the item
	for (int i = 0; i< numberOfItemsOnScreen; i++)
		createItem(p_fruit, p_world);
	
	//reset the score
	playerScore = 0;
	printf("Reset Complete\n");	
	
}


/*
Cleans up the class
*/
void cleanUp()
{

	CGImageRelease(emptyImage); 
	CGImageRelease(starImage);
	CGImageRelease(snakeImage);
	CGImageRelease(appleImage);
	CGImageRelease(strawberryImage);
	
}

*/